/***************************************************/
/*            Define Icon Menu                     */
/*  This module was contributed by Andre Hentz     */
/* andre@lcmi.ufsc.br. Blame all bugs on him!      */
/***************************************************/ 
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <unistd.h>
#include <stdio.h>

/*
 * Extern stuff
 */ 
extern Atom  XA_TYCOON;
extern Atom  XA_KILL_ACTIVE;
extern Atom  XA_ADD_ICON;
extern Atom  XA_EDIT_ICON;
extern Atom  XA_KILL;
extern Atom  XA_ALIGN;
extern Atom  XA_FREEZE;
extern Atom  XA_UNFREEZE;
extern Atom  XA_LOWER;
extern Atom  XA_RAISE;

Widget IconPopup;
extern Widget toplevel;
extern void UpdateRc ();

void CreateIconPopup();
void RemoveIconCb (Widget item, XtPointer lArg, XtPointer N);
void NewIconCb (Widget item, XtPointer lArg, XtPointer N);
void EditIconCb (Widget item, XtPointer lArg, XtPointer N);
void LowerIconCb (Widget item, XtPointer lArg, XtPointer N);
void RaiseIconCb (Widget item, XtPointer lArg, XtPointer N);
void UnfreezeIconCb (Widget item, XtPointer lArg, XtPointer N);
void FreezeIconCb (Widget item, XtPointer lArg, XtPointer N);
void ShutdownCb (Widget item, XtPointer lArg, XtPointer N);
void HandleClientMsg (Widget item, Widget junk, XEvent * event, Boolean* flag);
typedef struct{
	String name;
	String label;	
	XtCallbackProc callback;
} MenuItemRec;

void CreateIconPopup()
{
	Widget item;
	MenuItemRec items[] = {
		{"line0",  NULL			, NULL},
		{"item1", "New"			, NewIconCb},
		{"item4", "Edit"  		, EditIconCb},
		{"item5", "Remove"		, RemoveIconCb},
		{"line1",  NULL			, NULL},
		{"item6", "Lower"       , LowerIconCb},
		{"item7", "Raise"       , RaiseIconCb},
		{"item8", "Unfreeze"    , UnfreezeIconCb},
		{"item9", "Freeze"      , FreezeIconCb},
		{"line2",  NULL			, NULL},
		{"item2", "Quit"    	, ShutdownCb},
	};
	int i, n_items = XtNumber(items);
	
    IconPopup = XtVaCreatePopupShell("IconMenu", 
					simpleMenuWidgetClass, toplevel, 
					XtNlabel, "Tycoon",
					NULL );
	
	for(i=0;i<n_items;i++)
		if (!items[i].callback)
		    item = XtVaCreateManagedWidget(items[i].name,
					      smeLineObjectClass, IconPopup,
					      NULL);			
		else
		{			
		    item = XtVaCreateManagedWidget(items[i].name,
					      smeBSBObjectClass, IconPopup,
					      XtNlabel, items[i].label,
					      NULL);
		    XtAddCallback(item, XtNcallback,
			    	items[i].callback, (XPointer)NULL );
		}
}

/********************************************************/
/*            Icon Menu Callbacks                       */
/********************************************************/
void RemoveIconCb (Widget item, XtPointer lArg, XtPointer N)
{
  static XEvent event;
  Widget junk;
  Boolean * flag;

  event.xclient.message_type = XA_KILL_ACTIVE;
  HandleClientMsg (item, junk, &event, flag);  
}
void NewIconCb (Widget item, XtPointer lArg, XtPointer N)
{	
#ifdef WILL_FIX_IT
   long pid;

	if ((pid = fork()) == -1) 
	    fprintf(stderr, "Tycoon can't fork");
	else 
	{
    	if (!pid) 
		{	/* child's thread */
		      execlp("tyman", "tyman", "-add",NULL);
		      perror("Exec failed");
		      exit(1);
		} 
		else 
		{	/* parent's thread */
    	}
	}
#endif
   system("tyman -add&");
}
void EditIconCb (Widget item, XtPointer lArg, XtPointer N)
{
#ifdef WILL_FIX_IT
   long pid;
	
    UpdateRc();
	
	if ((pid = fork()) == -1) 
	    fprintf(stderr, "Tycoon can't fork");
	else 
	{
    	if (!pid) 
		{	/* child's thread */
		      execlp("tyman", "tyman", "-edit", NULL);
		      perror("Exec failed");
		      exit(1);
		} 
		else 
		{	/* parent's thread */
    	}
	}
#endif
   system("tyman -edit&");
}
void LowerIconCb (Widget item, XtPointer lArg, XtPointer N)
{
  static XEvent event;
  Widget junk;
  Boolean * flag;

  event.xclient.message_type = XA_LOWER;
  HandleClientMsg (item, junk, &event, flag);
}
void RaiseIconCb (Widget item, XtPointer lArg, XtPointer N)
{
  static XEvent event;
  Widget junk;
  Boolean * flag;

  event.xclient.message_type = XA_RAISE;
  HandleClientMsg (item, junk, &event, flag);
}
void UnfreezeIconCb (Widget item, XtPointer lArg, XtPointer N)
{
  static XEvent event;
  Widget junk;
  Boolean * flag;

  event.xclient.message_type = XA_UNFREEZE;
  HandleClientMsg (item, junk, &event, flag);
       
}
void FreezeIconCb (Widget item, XtPointer lArg, XtPointer N)
{
  static XEvent event;
  Widget junk;
  Boolean * flag;

  event.xclient.message_type = XA_FREEZE;
  HandleClientMsg (item, junk, &event, flag);
}
void ShutdownCb (Widget item, XtPointer lArg, XtPointer N)
{
     static XEvent event;
  Widget junk;
  Boolean * flag;

  event.xclient.message_type = XA_KILL;
  HandleClientMsg (item, junk, &event, flag);
}



